package com.persagy.dmp.report.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.report.common.criteria.CriteriaUtils;
import com.persagy.dmp.report.common.criteria.JacksonCriteria;
import com.persagy.dmp.report.entity.BaseEntity;
import com.persagy.dmp.report.entity.QReportConfig;
import com.persagy.dmp.report.entity.ReportConfig;
import com.persagy.dmp.report.repository.ReportConfigRepository;
import com.querydsl.core.Tuple;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Optional;

/**
 * @author fengyanjie
 */
@Service
public class ReportConfigService implements BaseService {

    @Autowired
    protected ReportConfigRepository reportConfigRepository;

    @Autowired
    private CriteriaUtils criteriaUtils;

    @Override
    public List<BooleanExpression> parse(ObjectNode criteria) {
        List<BooleanExpression> exps = new LinkedList<>();
        QReportConfig qt = QReportConfig.reportConfig;
        exps.addAll(CriteriaUtils.parse(qt.calculated, criteria.get("calculated")));
        exps.addAll(CriteriaUtils.parse(qt.tableName, criteria.get("tableName")));
        exps.addAll(CriteriaUtils.parse(qt.mappingName, criteria.get("mappingName")));
        exps.addAll(CriteriaUtils.parse(qt.groupColumn, criteria.get("group")));
        return exps;
    }

    public PagedResponse<ReportConfig> query(JacksonCriteria criteria) {
        return criteriaUtils.query(QReportConfig.reportConfig, this::parse, criteria);
    }

    @Override
    public List<BaseEntity> getData(PagedResponse<Tuple> tuplePagedResponse, List<JacksonCriteria.Calculated> calculatedColumns) {
        List<BaseEntity> list = new ArrayList<>();
        QReportConfig qt = QReportConfig.reportConfig;
        tuplePagedResponse.getData().forEach(item -> {
            ReportConfig entity = new ReportConfig();
            entity.setCalculated(item.get(qt.calculated));
            entity.setTableName(item.get(qt.tableName));
            entity.setMappingName(item.get(qt.mappingName));
            entity.setGroupColumn(item.get(qt.groupColumn));
            entity.setTableGroup(item.get(qt.tableGroup));
            list.add(entity);
        });
        return list;
    }

    public MapResponse create(ReportConfig param) {
        MapResponse response = new MapResponse();

        String calculated = param.getCalculated();
        if (calculated == null) {
            response.setFail("calculated is required");
            return response;
        }

        long count = reportConfigRepository.count(QReportConfig.reportConfig.calculated.eq(calculated));
        if (count > 0) {
            response.setFail("calculated is exists");
            return response;
        }
        reportConfigRepository.save(param);
        response.add("calculated", param.getCalculated());
        return response;
    }

    public MapResponse update(ReportConfig param) {
        MapResponse response = new MapResponse();

        String calculated = param.getCalculated();
        if (calculated == null) {
            response.setFail("calculated is required");
            return response;
        }

        Optional<ReportConfig> one = reportConfigRepository.findOne(QReportConfig.reportConfig.calculated.eq(calculated));
        if (!one.isPresent()) {
            response.setFail("calculated is not exists");
            return response;
        }
        reportConfigRepository.save(param);
        response.add("calculated", param.getCalculated());
        return response;
    }

    public MapResponse delete(ReportConfig param) {
        MapResponse response = new MapResponse();

        String calculated = param.getCalculated();
        if (calculated == null) {
            response.setFail("calculated is required");
            return response;
        }

        reportConfigRepository.deleteById(calculated);
        return response;
    }

    public MapResponse groupbyColumns(String tableName) {
        MapResponse response = new MapResponse();
        String groupbyColumns = reportConfigRepository.groupbyColumns(tableName);
        response.add("data", groupbyColumns.split(","));
        return response;
    }

    public MapResponse calculated(String tableName) {
        MapResponse response = new MapResponse();
        List<String> calculated = reportConfigRepository.calculated(tableName);
        response.add("data", calculated);
        return response;
    }
}
